////////////////////////////////////////////////////////////
// WinSock debug utility								  //
// (C) 2002 by Thomas Bleeker							  //
// www.MadWizard.org									  //
////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "common.h"
#include "commctrl.h"
#include "commdlg.h"
#include "winsock2.h"
#include "windowsx.h"

HINSTANCE	hInstance;




HWND hMainTab, hStatus;
HWND hRecvEdit, hSendEdit;

HANDLE hDebugThread;
DWORD  debugThreadID;

HMENU  hTabRightMenu;
extern PROCESS_INFORMATION piInfo;
int		iTabRightClicked;
extern unsigned long __stdcall DebugThread (void *);

OPENFILENAME openFileName;

char	programFileName[270];



void RegisterClass(HINSTANCE hInstance);
bool InitInstance();
void ProcessResize(HWND hWnd, WPARAM wParam, LPARAM lParam);
bool CreateControls(HWND hWnd);


#define DATA_BUFFER_SIZE 4096

typedef struct
{
	DWORD	hSocket;						//socket handle
	char	recvBuffer[DATA_BUFFER_SIZE];	//received data buffer
	char	sendBuffer[DATA_BUFFER_SIZE];	//send data buffer
} DEBUG_SOCKET;


LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

void UpdateInfo(LPVOID pModifiedDebugSocket)
{
	TCITEM	tabItem;
	int selectedItem;

	DEBUG_SOCKET *debugSocket;

	selectedItem = SendMessage(hMainTab, TCM_GETCURSEL,0,0);
	if (selectedItem==-1)
	{
		SetWindowText(hRecvEdit,"");
		SetWindowText(hSendEdit,"");
		return;
	}
	
	tabItem.mask = TCIF_PARAM;
	
	SendMessage(hMainTab, TCM_GETITEM, selectedItem,(long)&tabItem);
	
	debugSocket = (DEBUG_SOCKET*)tabItem.lParam;

	if (debugSocket!=pModifiedDebugSocket && pModifiedDebugSocket!=0)
		return;
	
	LockWindowUpdate(hRecvEdit);
	SetWindowText(hRecvEdit, debugSocket->recvBuffer);
	if (GetFocus()!=hRecvEdit)
		SendMessage(hRecvEdit, WM_VSCROLL, SB_BOTTOM, 0);

	LockWindowUpdate(hSendEdit);
	SetWindowText(hSendEdit, debugSocket->sendBuffer);
	if (GetFocus()!=hSendEdit)
		SendMessage(hSendEdit, WM_VSCROLL, SB_BOTTOM, 0);
	LockWindowUpdate(0);

}


void ProcessResize(HWND hWnd, WPARAM wParam, LPARAM lParam)
{
	RECT statusRect;
	RECT tabRect;

	SendMessage(hStatus,WM_SIZE,0,wParam);
	GetWindowRect(hStatus, &statusRect);
	GetWindowRect(hMainTab, &tabRect);

	
	int tabHeight = tabRect.bottom - tabRect.top;
	int winWidth = lParam&0xFFFF;
	int midHeight = (lParam>>16) - tabHeight - (statusRect.bottom - statusRect.top);
	MoveWindow(hMainTab,0,0,winWidth,tabHeight,true);
	
	MoveWindow(hRecvEdit,0,tabHeight,winWidth,midHeight/2,true);
	MoveWindow(hSendEdit,0,midHeight/2+tabHeight,winWidth,midHeight/2+midHeight%2,true);
}

bool CreateControls(HWND hWnd)
{
	TC_ITEM tempItem;
	RECT	tempRect;	


	/*=---------------------------------
		Tab control
	----------------------------------=*/

	hMainTab = CreateWindowEx(NULL,"SysTabControl32",NULL,
					WS_CHILD|WS_VISIBLE|TCS_FLATBUTTONS|TCS_BUTTONS|TCS_HOTTRACK|
					WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
					0,0,100,10,hWnd,(HMENU)ID_MAIN_TAB,hInstance,0);
	
	SendMessage(hMainTab, WM_SETFONT, (long)GetStockObject(DEFAULT_GUI_FONT),0);
	tempItem.mask = TCIF_TEXT;
	tempItem.pszText = "W";
	SendMessage(hMainTab, TCM_INSERTITEM, 0, (long)&tempItem);
	SendMessage(hMainTab, TCM_GETITEMRECT, 0, (long)&tempRect);
	SendMessage(hMainTab, TCM_DELETEITEM, 0,0);
	MoveWindow(hMainTab,0,0,100,tempRect.bottom - tempRect.top,true);

	/*=---------------------------------
		Edit controls
	----------------------------------=*/
	
	hRecvEdit = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", NULL,
					WS_CHILD|WS_VISIBLE|WS_VSCROLL|ES_AUTOVSCROLL|
					ES_MULTILINE|ES_READONLY|
					WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
					0,50,100,100,hWnd,(HMENU)ID_RECV_EDIT,hInstance,0);
	
	hSendEdit = CreateWindowEx(WS_EX_CLIENTEDGE, "EDIT", NULL,
					WS_CHILD|WS_VISIBLE|WS_VSCROLL|ES_AUTOVSCROLL|
					ES_MULTILINE|ES_READONLY|
					WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
					100,50,100,100,hWnd,(HMENU)ID_RECV_EDIT,hInstance,0);
	HFONT monoFont = (HFONT) GetStockObject(ANSI_FIXED_FONT);
	SendMessage(hRecvEdit, WM_SETFONT, (long)monoFont, 0);
	SendMessage(hSendEdit, WM_SETFONT, (long)monoFont, 0);

	/*=---------------------------------
		StatusBar
	----------------------------------=*/
	
	hStatus = CreateStatusWindow(WS_CHILD|WS_VISIBLE|SBARS_SIZEGRIP|
				WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
				"WinSock hook debug tool. (C) 2002 by Thomas Bleeker [www.MadWizard.org].",
				hWnd,ID_STATUS);

	/*=---------------------------------
		Popup menu
	----------------------------------=*/
	
	hTabRightMenu = CreatePopupMenu();

	InsertMenu(hTabRightMenu, 0, MF_BYPOSITION|MF_STRING,IDM_CLOSETAB,"&Close");
	InsertMenu(hTabRightMenu, 1, MF_BYPOSITION|MF_STRING,IDM_CLOSEALL,"Close &all");
	
	return true;
}

void CloseAllTabs()
{
	TCITEM tabItem;

	bool allClosed = true;

	int	tabCount = SendMessage(hMainTab, TCM_GETITEMCOUNT,0,0);

	tabItem.mask = TCIF_PARAM;

	int iFirstDeletable = 0;
	for (int i=0;i<tabCount;i++)
	{
		SendMessage(hMainTab, TCM_GETITEM, iFirstDeletable, (long)&tabItem);
		DEBUG_SOCKET *debugSocket = (DEBUG_SOCKET*)tabItem.lParam;
			
		if (debugSocket && (int)debugSocket->hSocket==-1)
		{
			SendMessage(hMainTab, TCM_DELETEITEM, iFirstDeletable,0);
			HeapFree(GetProcessHeap(),0,(void*)debugSocket);
		}
		else
		{
			iFirstDeletable++;
			allClosed = false;
		}
	}
	if (!allClosed)
		MessageBeep(0);

	UpdateInfo(NULL);
}

void CloseTab()
{
	TCITEM tabItem;

	tabItem.mask = TCIF_PARAM;

	SendMessage(hMainTab, TCM_GETITEM, iTabRightClicked, (long)&tabItem);
	DEBUG_SOCKET *debugSocket = (DEBUG_SOCKET*)tabItem.lParam;
		
	if ((int)debugSocket->hSocket==-1)
	{
		SendMessage(hMainTab, TCM_DELETEITEM, iTabRightClicked,0);
		HeapFree(GetProcessHeap(),0,(void*)debugSocket);
		UpdateInfo(NULL);
	}
	else
	{
		MessageBeep(0);
	}

}


int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	MSG msg;

	RegisterClass(hInstance);

	if (!InitInstance()) 
		return FALSE;


	while (GetMessage(&msg, NULL, 0, 0)) 
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return msg.wParam;
}




void RegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= 0;
	wcex.lpfnWndProc	= (WNDPROC)WndProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= LoadIcon(hInstance, (LPCTSTR)IDI_WSHOOK);
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_BTNFACE+1);
	wcex.lpszMenuName	= MAKEINTRESOURCE(IDR_MENU1);
	wcex.lpszClassName	= "wshook32";
	wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_SMALL);
	
	RegisterClassEx(&wcex);
}

//

bool InitInstance()
{
   HWND hWnd;

   hWnd = CreateWindow("wshook32", "WinSock hook debug tool",
	   WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN|WS_CLIPSIBLINGS,
      CW_USEDEFAULT, 0, CW_USEDEFAULT, 0, NULL, NULL, hInstance, NULL);

   if (!hWnd)
   {
      return FALSE;
   }

   ShowWindow(hWnd, SW_SHOW);
   UpdateWindow(hWnd);

   return TRUE;
}



DEBUG_SOCKET* findDebugSocket(DWORD hSocket)
{
	TCITEM		 tabItem;
	DEBUG_SOCKET *debugSocket;
	tabItem.mask = TCIF_PARAM;

	for (int i=0;
		 i<SendMessage(hMainTab,TCM_GETITEMCOUNT,0,0);
		 i++)
	{
		SendMessage(hMainTab,TCM_GETITEM,i,(long)&tabItem);
		debugSocket = (DEBUG_SOCKET*)tabItem.lParam;
		if (debugSocket->hSocket==hSocket)
			return debugSocket;
	}
	return NULL;
}

void HandleTabRightClick(HWND hWnd)
{
	TCHITTESTINFO tempHitTest;

	DWORD position = GetMessagePos();

	tempHitTest.pt.x = GET_X_LPARAM(position);
	tempHitTest.pt.y = GET_Y_LPARAM(position);

	ScreenToClient(hMainTab, &tempHitTest.pt);
	
	tempHitTest.flags = TCHT_ONITEM;

	iTabRightClicked  = SendMessage(hMainTab, TCM_HITTEST, 0, (long)&tempHitTest);

	if (iTabRightClicked!=-1)
	{
		TrackPopupMenu(hTabRightMenu, 0, GET_X_LPARAM(position), 
						GET_Y_LPARAM(position), NULL, hWnd, NULL);
	}
}


LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{

	switch (message) 
	{
	case WM_CREATE:
		CreateControls(hWnd);
		hDebugThread = CreateThread(NULL,NULL,&DebugThread,&programFileName, 0,&debugThreadID);
		return true;
	case WM_SIZE:
		ProcessResize(hWnd, wParam, lParam);
		return true;
	case WM_NOTIFY:
		if ((int)wParam==ID_MAIN_TAB)
		{
			switch(((LPNMHDR)lParam)->code)
			{
			case TCN_SELCHANGE:
				UpdateInfo(0);
				break;
			case NM_RCLICK:
				HandleTabRightClick(hWnd);
				break;
			}
		}
		return false;
	case WM_COMMAND:
		if (HIWORD(wParam)==0)
		{
			switch(wParam)
			{
			case IDM_CLOSETAB:
				CloseTab();
				break;
			case IDM_CLOSEALL:
				CloseAllTabs();
				break;
			case IDM_EXIT:
				SendMessage(hWnd,WM_CLOSE,0,0);
				break;
			case IDM_ABOUT:
				MessageBox(NULL, 
							"WinSock hook debug utility\n"
							"Beta version 1.0\n\n"
							"(C) 2002 by Thomas Bleeker\n"
							"www.MadWizard.org",
							"About wshook...",MB_ICONINFORMATION);
				break;
			default:
				return true;
			}
			return 0;
		}
		return true;
	case WM_DESTROY:
		CloseAllTabs();
		PostQuitMessage(0);
		break;
	default:
		return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}



void AddToBuffer(char *dest, LPVOID lpSource, int addLength, HANDLE hProcess)
{
	DWORD	bytesRead;
	int	currentLength = lstrlen(dest);
	
	int wantedSize = (addLength > (DATA_BUFFER_SIZE-1)) ?
					DATA_BUFFER_SIZE - 1 : addLength;

	int freeSize =  (DATA_BUFFER_SIZE-1-currentLength);

	if (freeSize<wantedSize)
	{
		// shift buffer
		for (int i=0;i<(currentLength-(wantedSize-freeSize));i++)
			dest[i] = dest[i+(wantedSize-freeSize)];
		
		freeSize = wantedSize;
		currentLength = (DATA_BUFFER_SIZE-1) - wantedSize;
	}
	
	ReadProcessMemory(hProcess, (LPVOID)((long)lpSource + addLength - wantedSize), (char*)&dest[currentLength],wantedSize,&bytesRead);

	for (int i=currentLength;i<(currentLength+wantedSize);i++)
	{
		if (dest[i]==0)
			dest[i]='.';
	}
	dest[currentLength+wantedSize] = 0;
}

void AddDebugSocket(long hSocket)
{
	DEBUG_SOCKET *pDebugSocket;
	char	buffer16[16];

	TCITEM tabItem;

	pDebugSocket = 
		(DEBUG_SOCKET*)HeapAlloc(GetProcessHeap(),0,sizeof(DEBUG_SOCKET));

	pDebugSocket->hSocket = hSocket;
	pDebugSocket->recvBuffer[0] = 0;
	pDebugSocket->sendBuffer[0] = 0;

	tabItem.mask = TCIF_PARAM|TCIF_STATE|TCIF_TEXT;
	tabItem.dwState = TCIS_BUTTONPRESSED;
	wsprintf(buffer16,"0x%X",hSocket);
	tabItem.pszText = buffer16;
	tabItem.lParam = (long)pDebugSocket;
	
	SendMessage(hMainTab, TCM_INSERTITEM, 
		SendMessage(hMainTab,TCM_GETITEMCOUNT,0,0),
		(long)&tabItem);
}

	
// context is context AFTER function call (eax=ret value)
void HandleHook(HANDLE hProcess, ACTIVE_HOOK *activeHook, CONTEXT *context)
{

	ULONG	retVal; //return value of API call
	retVal  = (ULONG)context->Eax;
	


	DEBUG_SOCKET *pDebugSocket;


	switch(activeHook->hookIndex)
	{
	case HOOK_INDEX_WRECV:
	case HOOK_INDEX_RECV:
	case HOOK_INDEX_ORECV:
		pDebugSocket = 
			findDebugSocket(activeHook->params[0]); //1st param is socket handle

		if (pDebugSocket)
		{
			if (retVal!=SOCKET_ERROR && retVal!=0)
			{
				AddToBuffer(pDebugSocket->recvBuffer, (char*)activeHook->params[1],
						retVal, hProcess);
				UpdateInfo(pDebugSocket);
			}

		}
		
		break;


	case HOOK_INDEX_WSEND:
	case HOOK_INDEX_OSEND:
	case HOOK_INDEX_SEND:
	
		pDebugSocket = 
			findDebugSocket(activeHook->params[0]); //1st param is socket handle

		if (pDebugSocket)
		{
			if (retVal!=SOCKET_ERROR && retVal!=0)
			{
				AddToBuffer(pDebugSocket->sendBuffer, (char*)activeHook->params[1],
						retVal, hProcess);
				UpdateInfo(pDebugSocket);
			}
		}
		
		break;
	case HOOK_INDEX_WSOCKETA:
	case HOOK_INDEX_WSOCKETW:
	case HOOK_INDEX_WACCEPT:
	case HOOK_INDEX_OSOCKET:
	case HOOK_INDEX_SOCKET:
	case HOOK_INDEX_ACCEPT:
	case HOOK_INDEX_OACCEPT:

		if (retVal!=INVALID_SOCKET)
			AddDebugSocket(retVal);

		break;
	case HOOK_INDEX_CLOSE:
	case HOOK_INDEX_OCLOSE:
		pDebugSocket = 
			findDebugSocket(activeHook->params[0]); //1st param is socket handle
		if (retVal==0 && pDebugSocket)
		{
			pDebugSocket->hSocket = -1;
		}
		break;

	}
}


void entrypoint()
{
	hInstance = GetModuleHandle(NULL);
	

	openFileName.lStructSize = sizeof(openFileName);
	openFileName.hInstance = hInstance;
	openFileName.lpstrFilter = "Programs (*.exe)\0*.exe\0";
	openFileName.lpstrTitle = "Select a program to debug";
	openFileName.Flags = OFN_FILEMUSTEXIST|OFN_PATHMUSTEXIST;
	openFileName.lpstrFile = programFileName;
	openFileName.nMaxFile = 270;
	
	if (!GetOpenFileName(&openFileName))
		ExitProcess(0);
	

	InitCommonControls();

	WinMain(hInstance, NULL, NULL, SW_SHOW);
	ExitProcess(0);
}


